package org.tbasket.jwt

import zio.*
import zio.stream.*
import zio.http.*
import zio.http.ServerConfig.LeakDetectionLevel
import zio.http.model.Method

object Main extends ZIOAppDefault {
  
  private val app = Http.collectZIO[Request] {
    case r@Method.POST -> !! / "/jwt" =>
      JwtGeneration.generateTokenResponse(r)
    
  }
  
  val run =
    ZIO.serviceWithZIO[ZIOAppArgs] { args =>
      for
        port <- ZIO.attempt(args.getArgs)
          .map(_.head)
          .mapError(_ => Console.printError("Must provide at least one argument"))
          .map(_.toInt)
          .mapError(_ => Console.printError("Port is not a number"))
          .filterOrFail(p => 0 < p && p < 65563)(Console.printError("Port is out of range"))
      
      yield
        val config = ServerConfig.default
          .port(port)
          .leakDetection(LeakDetectionLevel.PARANOID)
        
        val configLayer = ServerConfig.live(config)
        (Server.install(app) *> Console.printLine("JWT AppToken open on port") *> ZIO.never)
          .provideSome(configLayer, Server.live)
    }
  
}
